/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "oirSupport.h"
#include "fitsio.h"
#include "imaging_fdata.h"

/**********************************************************
 * Usage:
 *    oirRescaleSpectrum templateFile photoFile outFile 
 *
 *    templateFile contains unscaled spectrum with all your favorite little bumps
 *    photoFile contains a crude spectrum which is correctly calibrated and normalized
 *
 *    Output looks like templateFile but has same 0th and 1st moments as photoFile
 *
 ************************************************************* */
void Usage() {
   printf("Usage:\n");
   printf("   oirRescaleSpectrum templateFile  photoFile outFile \n\n");
   printf("   template = accurate detailed but unscaled spectrum\n");
   printf("   photoFile = crude (e.g. poor resolution) but correctly calibrated spectrum\n");
   printf("   outFile = spectrum with details like template but 0th and 1st moments like photo\n");
   printf("  \n");
}

int main (int argc, char** argv) {

   int i, j, ierr, iSpec;
   int nx;
   int ix, iy;
   float x,y, nx2;


   char outFileName[130];
   char *bang="!";

   char history[80];

   fitsfile *photoFile;
   fitsfile *tempFile;
   fitsfile *outFile;

   imaging_fdata_table_def *photodata;
   imaging_fdata_table_def *tempdata;
   imaging_fdata_table_def *outdata;

   imaging_detector_table_def *DetectorDef;

   imaging_fdata_row *photoData;
   imaging_fdata_row *tempData;

   float photoMom[2];
   float templMom[2][2];
   float fitCoeff[2];

   float denom, num1, num2;
   ierr = 0;

   if (argc < 4) {
      Usage();
      return 1;
   }
   printf("      ****** oirRescaleSpectra ******\n");
/*
         loop over input rows get input data 
*/
   i = fits_open_file(&tempFile, argv[1], READONLY, &ierr);
   if (ierr != 0) {
      printf("Failed to open template data: error: %s %i\n", argv[1], ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   i = fits_open_file(&photoFile, argv[2], READONLY, &ierr);
   if (ierr != 0) {
      printf("Failed to open photo data: error: %s %i\n", argv[2], ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   DetectorDef = read_imaging_detector_table(tempFile, &ierr);
   nx = DetectorDef->naxis[0][0];

   strcpy(outFileName,"!");
   strcat(outFileName, argv[3]);
   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr != 0) {
      printf("Failed to open output data: error: %s %i\n", argv[3], ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   i = fits_copy_header(photoFile, outFile, &ierr);
   tempdata   = open_imaging_fdata_table(tempFile, &ierr);
   tempData   = make_imaging_fdata_row(tempdata);
   photodata  = open_imaging_fdata_table(photoFile, &ierr);
   photoData  = make_imaging_fdata_row(photodata);
   outdata    = make_imaging_fdata_table_def (outFile, DetectorDef, 1, &ierr);

/*
      loop over 6 regions ; read input data
      doing best least sq fit of template to photo including
      constant multiplier and linear x-dependence
      write output data
*/
   nx2 = (float)(.5*nx);
   for (iSpec=1;iSpec<=6;iSpec++) {
      for (i=0;i<2;i++) {
         photoMom[i] = 0.;
         for (j=0;j<2;j++) templMom[i][j] = 0.;
      }
      i=read_imaging_fdata_table(tempFile, tempdata, tempData, iSpec, &ierr);
      if (ierr !=0) {
         printf("ERROR %i reading template data line %i in file %s\n",
            ierr, iSpec, argv[1]);
         return 1;
      }
      i=read_imaging_fdata_table(photoFile, photodata, photoData, iSpec, &ierr);
      if (ierr !=0) {
         printf("ERROR %i reading photo data line %i in file %s\n",
            ierr, iSpec, argv[2]);
         return 1;
      }

      for (ix=0;ix<nx;ix++) {
         x = (float)(ix-nx2);
         photoMom[0] += photoData->data[0][ix] * tempData->data[0][ix];
         photoMom[1] += photoData->data[0][ix] * tempData->data[0][ix] * x;
         templMom[0][0] += tempData->data[0][ix] * tempData->data[0][ix];
         templMom[0][1] += tempData->data[0][ix] * tempData->data[0][ix] * x;
         templMom[1][0] = templMom[0][1];
         templMom[1][1] += tempData->data[0][ix] * tempData->data[0][ix]*x*x;
      } /* end of accumulate moments */
/*
        solve for linear coefficients
*/
      denom = templMom[0][0] * templMom[1][1] - templMom[1][0]*templMom[0][1];
      fitCoeff[0] = (photoMom[0]*templMom[1][1] - photoMom[1]*templMom[1][0])/denom;
      fitCoeff[1] = (photoMom[1]*templMom[0][0] - photoMom[0]*templMom[0][1])/denom;
/*
        replace template by scaled template
*/
      for (ix=0;ix<nx;ix++) tempData->data[0][ix] = tempData->data[0][ix] *
         (fitCoeff[0] + (ix-nx2)* fitCoeff[1]);
      i = write_imaging_fdata_table(outFile, outdata, tempData, iSpec, &ierr);
   }  /* loop over 6 rows */
/*
      just copy last 6 rows
*/
   for (iSpec=7;iSpec<=12;iSpec++) {
      i=read_imaging_fdata_table(photoFile, photodata, photoData, iSpec, &ierr);
      if (ierr !=0) {
         printf("ERROR %i reading photo data line %i in file %s\n",
            ierr, iSpec, argv[2]);
         return 1;
      }
      i = write_imaging_fdata_table(outFile, outdata, photoData, iSpec, &ierr);
   }
/*
      clean up
*/
   i = close_imaging_fdata_table(tempFile, tempdata, &ierr);
   i = close_imaging_fdata_table(photoFile, photodata, &ierr);
   i = close_imaging_fdata_table(outFile, outdata, &ierr);
   kill_imaging_fdata_row(tempData);
   kill_imaging_fdata_row(photoData);
   return 0;
}
